/*
 * pclt.h
 */
#ifndef PCLT_H_
#define PCLT_H_

struct pclt;

enum {
	CLIENT_DISCONNECTED,		/* Non-waiting state */
	CLIENT_CONNECTING,
	CLIENT_CONNECTED,		/* Non-wating state */
	CLIENT_INPUTS_WAIT,
	CLIENT_OUTPUTS_WAIT,
	CLIENT_OUTPUTS_WRITE,
	CLIENT_INPUTS_WRITE,
	CLIENT_PREPARE_READ_WAIT,
	CLIENT_PREPARE_WRITE_WAIT,
	CLIENT_END_READ_WAIT,
	CLIENT_END_WRITE_WAIT,
};

typedef int (*pclt_enum_callback_t)(const char *sockfile, int state, int status);

/* Initialize */
void pclt_init();

/* pclt_new - create new connection to server
 *
 * sock:		path to PF_UNIX server socket
 * timeout:		response timeout in msecs
 * inputs:		size of inputs image (I) for this connection in bytes
 * outputs:		size of outputs image (Q) for this connection in bytes
 * mode:		image byte order (le, be or native)
 * side:		master or slave
 */
struct pclt *pclt_new(const char *sock, int timeout, size_t inputs, size_t outputs, pi_mode_t mode, pi_side_t side);

int pclt_add_par(struct pclt *pclt, struct par_struct *par, par_dir_t dir, unsigned int offset, unsigned char bit);

/* pclt_connect - initiate connections to all servers */
void pclt_connect();

/* pclt_try_reconnect - try to connect to disconnected servers */
void pclt_try_reconnect();

/* pclt_enumerate -  enumerate all PI sessions and get status and state */
void pclt_enumerate(pclt_enum_callback_t callback);

/* ask server to prepare inputs image */
void pclt_prepare_inputs(pi_side_t side);

/* aks server to prepare outputs image */
void pclt_prepare_outputs(pi_side_t side);

/* read inputs from slave PI server */
void pclt_read_inputs();

/* write inputs to master PI server */
void pclt_write_inputs();

/* read outputs from master PI server */
void pclt_read_outputs();

/* write outputs to slave PI server */
void pclt_write_outputs();

/* Finalize.  */
void pclt_deinit();

#endif
